use based::ui::htmx::HTMXAttributes;
use based::ui::prelude::*;
use based::{
    auth::MaybeUser,
    request::{
        api::{vec_to_api, FallibleApiResponse},
        RequestContext, StringResponse,
    },
};
use maud::{html, Render};
use rocket::{get, State};
use serde_json::json;

use crate::pages::components::{Title, VerticalVideoGrid};
use crate::{library::Library, pages::components::video_element};

use super::{
    api_response,
    components::{render_page, video_element_wide},
};

#[get("/search?<query>&<offset>")]
pub async fn search(
    query: &str,
    offset: Option<i64>,
    library: &State<Library>,
) -> Option<serde_json::Value> {
    const NUM_OF_RESULTS: i64 = 20;

    // get start parameter for search result chunks
    let start = offset.unwrap_or(0);

    let mut video_matches = library.search_video(query, start, NUM_OF_RESULTS).await;

    Some(json!(vec_to_api(&mut video_matches).await))
}

#[get("/latest.json")]
pub async fn latest_api(library: &State<Library>) -> FallibleApiResponse {
    let videos = library.get_newly_added(20).await;
    let vid_api = vec_to_api(&videos).await;
    return Ok(serde_json::json!(vid_api));
}

#[get("/latest")]
pub async fn latest_page(
    ctx: RequestContext,
    library: &State<Library>,
    user: MaybeUser,
) -> StringResponse {
    let videos = library.get_newly_added(20).await;
    let video_elements = html! {
        @for mut vid in videos {
            ( video_element_wide(&mut vid).await );
        };
    };

    let content = Div()
        .vanish()
        .push(Title("Recent videos"))
        .push(Padding(video_elements).all(ScreenValue::_6))
        .render();

    render_page(ctx, content, "Recent videos", user.into()).await
}

#[get("/d/<dir>")]
pub async fn dir_page(
    ctx: RequestContext,
    dir: &str,
    library: &State<Library>,
    user: MaybeUser,
) -> StringResponse {
    if dir.ends_with(".json") {
        let dir_videos = library
            .get_directory_videos(dir.split_once(".json").map(|x| x.0).unwrap_or_default())
            .await;
        return api_response(&json!(vec_to_api(&dir_videos).await));
    }

    let dir_videos = library.get_directory_videos(dir).await;
    let video_elements = html! {
        @for mut vid in dir_videos {
            ( video_element_wide(&mut vid).await );
        };
    };

    let content = Div()
        .vanish()
        .push(Title(dir.to_string()))
        .push(Padding(video_elements).all(ScreenValue::_6))
        .render();

    render_page(ctx, content, dir, user.into()).await
}

#[get("/")]
pub async fn index_page(
    ctx: RequestContext,
    library: &State<Library>,
    user: MaybeUser,
) -> StringResponse {
    let random_video_elements = html! {
        @for mut vid in library.get_random_videos(3).await {
            ( video_element(&mut vid).await );
        };
    };
    let newly_added_elements = html! {
        @for mut vid in library.get_newly_added(3).await {
            ( video_element(&mut vid).await );
        };
    };
    let directories = library.get_directories().await;

    let content = Div()
        .vanish()
        .push(Title("Random Videos"))
        .push(VerticalVideoGrid(random_video_elements))
        .push(Title(Link("/latest", "Latest Videos").use_htmx()))
        .push(VerticalVideoGrid(newly_added_elements))
        .push(Title("Directories:"))
        .push(
            Padding(
                Flex(Div().vanish().push_for_each(&directories, |dir: &_| {
                    Margin(
                        Padding(
                            Hover(Background(Cursor::Pointer.on(Nothing())).color(Purple::_600))
                                .on(Background(
                                    Rounded(
                                        Link(&format!("/d/{dir}"), Text(&dir).white()).use_htmx(),
                                    )
                                    .size(Size::Full),
                                )
                                .color(Purple::_500)),
                        )
                        .x(ScreenValue::_3)
                        .y(ScreenValue::_2),
                    )
                    .all(ScreenValue::_2)
                }))
                .wrap(Wrap::Wrap),
            )
            .all(ScreenValue::_10),
        )
        .render();

    render_page(ctx, content, "WatchDogs", user.into()).await
}
